package fr.eni.ep3jasp;

import org.aopalliance.aop.Advice;

import org.aopalliance.intercept.MethodInterceptor;
import org.aopalliance.intercept.MethodInvocation;

import org.springframework.aop.Advisor;
import org.springframework.aop.ClassFilter;
import org.springframework.aop.Pointcut;
import org.springframework.aop.framework.ProxyFactory;
import org.springframework.aop.support.DefaultPointcutAdvisor;
import org.springframework.aop.support.StaticMethodMatcherPointcut;

import java.lang.reflect.Method;


/**
 * Exemple sur l'AOP
 *
 * @author $author$
 * @version $Revision$
  */
public class Main {
  /**
   * Point d'entrée
   *
   * @param args Les arguments de la ligne de commande
   */
  public static void main(String[] args) {
    PojoUn one = new PojoUn();
    PojoDeux two = new PojoDeux();

    PojoUn proxyUn;
    PojoDeux proxyDeux;

    Pointcut pc = new SimpleStaticPointcut();
    Advice advice = new SimpleAdvice();
    Advisor advisor = new DefaultPointcutAdvisor(pc, advice);

    ProxyFactory pf = new ProxyFactory();
    pf.addAdvisor(advisor);
    pf.setTarget(one);
    proxyUn = (PojoUn) pf.getProxy();

    pf = new ProxyFactory();
    pf.addAdvisor(advisor);
    pf.setTarget(two);
    proxyDeux = (PojoDeux) pf.getProxy();

    proxyUn.plus();
    proxyDeux.plus();

    proxyUn.moins();
    proxyDeux.moins();
  }
}


/**
 * Second Pojo
 *
 * @author $author$
 * @version $Revision$
  */
class PojoDeux {
  /**
   * Méthode plus
   */
  public void plus() {
    System.out.println("PojoDeux.plus");
  }

  /**
   * Méthode moins
   */
  public void moins() {
    System.out.println("PojoDeux.moins");
  }
}


/**
 * Premier Pojo
 *
 * @author $author$
 * @version $Revision$
  */
class PojoUn {
  /**
   * Méthode plus
   */
  public void plus() {
    System.out.println("PojoUn.plus");
  }

  /**
   * Méthode moins
   */
  public void moins() {
    System.out.println("PojoUn.moins");
  }
}


/**
 * Advice simple
 *
 * @author $author$
 * @version $Revision$
  */
class SimpleAdvice implements MethodInterceptor {
  /**
   * Invocation
   *
   * @param invocation la méthode à invoquer
   *
   * @return L'objet retourné
   *
   * @throws Throwable L'exception à transmettre
   */
  public Object invoke(MethodInvocation invocation) throws Throwable {
    System.out.println(">>>SimpleAdvice.Invoking " +
      invocation.getMethod().getName());

    Object retVal = invocation.proceed();
    System.out.println(">>>SimpleAdvice.Done");

    return retVal;
  }
}


/**
 * Pointcut statique simple
 *
 * @author $author$
 * @version $Revision$
  */
class SimpleStaticPointcut extends StaticMethodMatcherPointcut {
  /**
   * Méthode de comparaison
   *
   * @param method La méthode
   * @param cls La classe
   *
   * @return Résultat de la comparaison
   */
  public boolean matches(Method method, Class cls) {
    return ("plus".equals(method.getName()));
  }

  /**
   * Récupération du filtre de classe
   *
   * @return Le filtre de classe
   */
  public ClassFilter getClassFilter() {
    return new ClassFilter() {
        public boolean matches(Class cls) {
          return (cls == PojoUn.class);
        }
      };
  }
}
