package fr.eni.ep3jasp.aop2;

import org.aopalliance.aop.Advice;

import org.springframework.aop.AfterReturningAdvice;
import org.springframework.aop.framework.ProxyFactory;

import java.lang.reflect.Method;


/**
 * Interface de l'objet de démonstration Pojo
 *
 * @author $author$
 * @version $Revision$
  */
interface Pojo {
  /**
   * Méthode foo
   */
  public void foo();

  /**
   * Méthode bar
   */
  public void bar();
}


/**
 * Implémentation de l'objet de démonstration Pojo
 *
 * @author $author$
 * @version $Revision$
  */
class SimplePojo implements Pojo {
  /**
   * Méthode foo
   */
  public void foo() {
    System.out.println("foo");
    this.bar();
  }

  /**
   * Méthode bar
   */
  public void bar() {
    System.out.println("bar");
  }
}


/**
 * Méthode qui s'écécutera après la méthode interceptée
 *
 * @author $author$
 * @version $Revision$
  */
class RetryAdvice implements AfterReturningAdvice {
  /**
   * Code appélé avant de rendre la main
   *
   * @param returnValue Valuer à retourner
   * @param method Méthode à appeler
   * @param args Arguments de la méthode
   * @param target La cible
   *
   * @throws Throwable L'exception à transmettre
   */
  public void afterReturning(Object returnValue, Method method, Object[] args,
    Object target) throws Throwable {
    System.out.println("after " + method.getName());
  }
}


/**
 * Cet exemple montre la programmation par aspects en utilisant la configuration par java
 *
 * @author $author$
 * @version $Revision$
  */
public class Main {
  /**
   * Point d'entrée
   *
   * @param args Les arguments de la ligne de commande
   */
  public static void main(String[] args) {
    ProxyFactory factory = new ProxyFactory(new SimplePojo());
    Class<?> intf = Pojo.class;
    factory.addInterface(intf);

    Advice advice = new RetryAdvice();
    factory.addAdvice(advice);
    factory.setExposeProxy(true);

    Pojo pojo = (Pojo) factory.getProxy();

    pojo.foo();
    pojo.bar();
  }
}
