package fr.eni.ep3jasp.aop2;

import org.aopalliance.aop.Advice;

import org.springframework.aop.AfterReturningAdvice;
import org.springframework.aop.framework.ProxyFactory;

import java.lang.reflect.Method;


/**
 * L'interface du Pojo
 *
 * @author $author$
 * @version $Revision$
  */
interface Pojo {
  /**
   * Méthode foo
   */
  public void foo();

  /**
   * Méthode bar
   */
  public void bar();
}


/**
 * Implémentation du Pojo
 *
 * @author $author$
 * @version $Revision$
  */
class SimplePojo implements Pojo {
  /**
   * La méthode foo
   */
  public void foo() {
    System.out.println("foo");
    this.bar();
  }

  /**
   * La méthode bar
   */
  public void bar() {
    System.out.println("bar");
  }
}


/**
 * Advice qui s'exécute après le retour de la méthode interceptée
 *
 * @author $author$
 * @version $Revision$
  */
class RetryAdvice implements AfterReturningAdvice {
  /**
   * Après le retour
   *
   * @param returnValue valeur retournée
   * @param method Méthode appelée
   * @param args Argumens
   * @param target La cible
   *
   * @throws Throwable L'exception à transmettre
   */
  public void afterReturning(Object returnValue, Method method, Object[] args,
    Object target) throws Throwable {
    System.out.println("after " + method.getName());
  }
}


/**
 * Exemple utilisant l'AOP configuré par les fichiers xml
 *
 * @author $author$
 * @version $Revision$
  */
public class Main {
  /**
   * Point d'entrée
   *
   * @param args Les arguments de la ligne de commande
   */
  public static void main(String[] args) {
    ProxyFactory factory = new ProxyFactory(new SimplePojo());
    Class<?> intf = Pojo.class;
    factory.addInterface(intf);

    Advice advice = new RetryAdvice();
    factory.addAdvice(advice);
    factory.setExposeProxy(true);

    Pojo pojo = (Pojo) factory.getProxy();

    pojo.foo();
    pojo.bar();
  }
}
