package fr.eni;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.dbcp.BasicDataSource;
import org.springframework.beans.factory.BeanFactory;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.PreparedStatementCreator;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.datasource.SingleConnectionDataSource;
import org.springframework.jdbc.support.GeneratedKeyHolder;
import org.springframework.jdbc.support.KeyHolder;
import org.springframework.jdbc.support.rowset.SqlRowSet;

/**
 * Utilisation moins simple de Spring JDBC
 *
 * @author $author$
 * @version $Revision$
  */
@Slf4j
public class Complexe {
	public static void main(String[] args) throws SQLException {
		ClassPathXmlApplicationContext appContext;;
		
		appContext = new ClassPathXmlApplicationContext(
				"applicationContext.xml");
		
		BeanFactory factory = (BeanFactory) appContext;
		BasicDataSource dataSource = (BasicDataSource) factory
				.getBean("dataSource");
		ComplexeDao dao = (ComplexeDao) factory
				.getBean("complexeDao");

		JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
		jdbcTemplate.execute("drop TABLE IF EXISTS utilisateur");
		jdbcTemplate
				.execute("create table utilisateur (id serial, nom varchar, prenom varchar)");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('A', 'a')");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('B', 'b')");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('C', 'c')");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('D', 'd')");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('E', 'e')");
		jdbcTemplate
				.execute("insert into utilisateur (nom, prenom) values ('F', 'f')");

		final Utilisateur modele=new Utilisateur(1,"a","A");
		
		Utilisateur utilisateur=dao.batchRechercherParModele(modele);
		List<Utilisateur> utilisateurs =dao.batchRechTousParModele(modele);
		
		log.info("Utilisateurs:");
		utilisateurs.forEach(util -> log.info(util.toString()));

		final String INSERT_SQL = "insert into utilisateur (nom) values(?)";
		final String name = "Moi";
		
		KeyHolder keyHolder = new GeneratedKeyHolder();
		jdbcTemplate.update(
		    new PreparedStatementCreator() {
		        public PreparedStatement createPreparedStatement(Connection connection) throws SQLException {
		            PreparedStatement ps = connection.prepareStatement(INSERT_SQL, new String[] {"id"});
		            ps.setString(1, name);
		            return ps;
		        }
		    },
		    keyHolder);

		log.info("clé="+keyHolder.getKey());

		List<Utilisateur> us=new ArrayList<Utilisateur>();
		us.add(new Utilisateur(1,"z","Z"));
		us.add(new Utilisateur(2,"y","Y"));
		us.add(new Utilisateur(3,"x","X"));
		
		log.info("classic JDBC...");
		int[] res= dao.batchMaj0(us);
		String codes = Arrays.stream(res)
			    .mapToObj(Integer::toString)
			    .collect(Collectors.joining(",","[","]"));
		
		log.info("codes="+codes);
		
		List<Utilisateur> us3 = dao.getAll();
		log.info("Après modif:"+us3.toString());
		
		us=new ArrayList<Utilisateur>();
		us.add(new Utilisateur(1,"ffff","FFFF"));
		us.add(new Utilisateur(2,"gggg","GGGG"));
		
		log.info("BatchPreparedStatementSetter...");
		res= dao.batchMaj1(us);
		
		codes = Arrays.stream(res)
			    .mapToObj(Integer::toString)
			    .collect(Collectors.joining(",","[","]"));
		
		log.info("codes="+codes);
		
		us3 = dao.getAll();
		log.info("Après modif:"+us3.toString());
		
		us=new ArrayList<Utilisateur>();
		us.add(new Utilisateur(1,"rrrr","RRRR"));
		us.add(new Utilisateur(2,"tttt","TTTT"));
		
		log.info("SqlParameterSource...");
		res= dao.batchMaj2(us);
		
		codes = Arrays.stream(res)
			    .mapToObj(Integer::toString)
			    .collect(Collectors.joining(",","[","]"));
		
		log.info("codes="+codes);
		
		us3 = dao.getAll();
		log.info("Après modif:"+us3.toString());
		
	}

}
