package fr.eni;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.dbcp.BasicDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.BatchPreparedStatementSetter;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSourceUtils;
import org.springframework.stereotype.Repository;

@Slf4j
@Repository
public class ComplexeDao {

	private BasicDataSource dataSource;
	private JdbcTemplate jdbcTemplate;
	private NamedParameterJdbcTemplate namedParameterJdbcTemplate;

	@Autowired
	public void setDataSource(BasicDataSource dataSource) {
		log.info("par setter setDataSource");
		this.dataSource = dataSource;
		this.jdbcTemplate = new JdbcTemplate(dataSource);
	    this.namedParameterJdbcTemplate = new NamedParameterJdbcTemplate(dataSource);


	}
	public List<Utilisateur> getAll() {
		List<Utilisateur> utilisateurs = jdbcTemplate.query(
				"select nom, prenom, id from utilisateur",
				new RowMapper<Utilisateur>() {
					public Utilisateur mapRow(ResultSet rs, int rowNum)
							throws SQLException {
						Utilisateur utilisateur = new Utilisateur();
						utilisateur.setNom(rs.getString("nom"));
						utilisateur.setPrenom(rs.getString("prenom"));
						utilisateur.setId(rs.getInt("id"));
						return utilisateur;
					}
				});

		return utilisateurs;
	}
	public Utilisateur batchRechercherParModele(final Utilisateur utilisateur) {
		Utilisateur u2 = jdbcTemplate.queryForObject(
				"select nom, prenom from utilisateur where id = ?",
				new Object[] { 5 }, new RowMapper<Utilisateur>() {
					public Utilisateur mapRow(ResultSet rs, int rowNum)
							throws SQLException {
						Utilisateur utilisateur = new Utilisateur();
						utilisateur.setNom(rs.getString("nom"));
						utilisateur.setPrenom(rs.getString("prenom"));
						return utilisateur;
					}
				});
		System.out.println(u2.toString());
		return u2;
	}
	public List<Utilisateur> batchRechTousParModele(final Utilisateur utilisateur) {
		List<Utilisateur> utilisateurs = jdbcTemplate.query(
		"select nom, prenom from utilisateur",
		new RowMapper<Utilisateur>() {
			public Utilisateur mapRow(ResultSet rs, int rowNum)
					throws SQLException {
				Utilisateur utilisateur = new Utilisateur();
				utilisateur.setNom(rs.getString("nom"));
				utilisateur.setPrenom(rs.getString("prenom"));
				return utilisateur;
			}
		});
		return utilisateurs;
	}
	public int[] batchMaj0(final List<Utilisateur> utilisateurs) {
        List<Object[]> batch = new ArrayList<Object[]>();
        for (Utilisateur actor : utilisateurs) {
            Object[] values = new Object[] {
                    actor.getNom(),
                    actor.getPrenom(),
                    actor.getId()};
            batch.add(values);
        }
        int[] updateCounts = jdbcTemplate.batchUpdate(
                "update utilisateur set nom = ?, prenom = ? where id = ?",
                batch);
        return updateCounts;
	}
	
	public int[] batchMaj1(final List<Utilisateur> utilisateurs) {
		int[] updateCounts = jdbcTemplate.batchUpdate(
				"update utilisateur set nom = ?, "
						+ "prenom = ? where id = ?",
				new BatchPreparedStatementSetter() {
					public void setValues(PreparedStatement ps, int i)
							throws SQLException {
						ps.setString(1, utilisateurs.get(i).getNom());
						ps.setString(2, utilisateurs.get(i).getPrenom());
						ps.setLong(3, utilisateurs.get(i).getId());
					}

					public int getBatchSize() {
						return utilisateurs.size();
					}
				});
		return updateCounts;
	}
	


    public int[] batchMaj2(final List<Utilisateur> utilisateurs) {
        SqlParameterSource[] batch = SqlParameterSourceUtils.createBatch(utilisateurs.toArray());
        int[] updateCounts = namedParameterJdbcTemplate.batchUpdate(
                "update utilisateur set nom = :nom, prenom = :prenom where id = :id",
                batch);
        return updateCounts;
    }
	
}
