package fr.eni;

import java.sql.Types;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.SingleConnectionDataSource;
import org.springframework.jdbc.support.rowset.SqlRowSet;

/**
 * Exemple Spring JDBC difficulté moyenne
 *
 * @author $author$
 * @version $Revision$
  */
@Slf4j
public class Moyenne {
	public static void main(String[] args) {
		SingleConnectionDataSource dataSource = new SingleConnectionDataSource();
		dataSource.setDriverClassName("org.h2.Driver");
		dataSource.setUrl("jdbc:h2:~/test3");
		dataSource.setUsername("sa");
		dataSource.setPassword("");

		JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
		jdbcTemplate.execute("drop TABLE IF EXISTS utilisateur");
		jdbcTemplate.execute("create table utilisateur (id int, nom varchar, prenom varchar)");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (1, 'A', 'a')");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (2, 'B', 'b')");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (3, 'C', 'c')");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (4, 'D', 'd')");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (5, 'E', 'e')");
		jdbcTemplate.execute("insert into utilisateur (id, nom, prenom) values (6, 'F', 'f')");

        // Split up the array of whole names into an array of first/last names
		// On utilise les streams java 8 pour préparer les lignes à insérer
        List<Object[]> valeursFormatees = Arrays.asList("G g", "H h", "I i", "J j").stream()
                .map(enreg -> enreg.split(" "))
                .collect(Collectors.toList());

        // Use a Java 8 stream to print out each tuple of the list
        valeursFormatees.forEach(enreg -> log.info(String.format("Insersion enregistrement dans la BDD : %s %s", enreg[0], enreg[1])));

        // Uses JdbcTemplate's batchUpdate operation to bulk load data
        jdbcTemplate.batchUpdate("INSERT INTO utilisateur(nom, prenom) VALUES (?,?)", valeursFormatees);

        log.info("Recherche utilisateur nom > 'H':");
        jdbcTemplate.query(
                "SELECT id, nom, prenom FROM utilisateur WHERE nom > ?", new Object[] { "H" },
                (rs, rowNum) -> new Utilisateur(rs.getLong("id"), rs.getString("nom"), rs.getString("prenom"))
        ).forEach(utilisateur -> log.info(utilisateur.toString()));

		//la même chose que l'exemple précédent pas avec un objet Utilisateur, l'utilisation de lambok pour les accesseurs
        //et java 8 pour la programmation des interfaces fonctionnelles et des lambdas qui simplifie le code.
	
        List<Map<String, Object>> liste = jdbcTemplate.queryForList("select * from utilisateur");
        log.info(liste.toString());
        
		jdbcTemplate.execute("delete from utilisateur");
		jdbcTemplate.execute("drop table utilisateur");

		dataSource.destroy();
	}

}
