package fr.eni;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;

import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.datasource.SingleConnectionDataSource;
import org.springframework.jdbc.support.rowset.SqlRowSet;

final class UtilisateurMapper implements RowMapper<Utilisateur> {

    public Utilisateur mapRow(ResultSet rs, int rowNum) throws SQLException {
    	Utilisateur utilisateur = new Utilisateur();
        utilisateur.setNom(rs.getString("nom"));
        utilisateur.setPrenom(rs.getString("prenom"));
        return utilisateur;
    }
}

/**
 * Exemple Spring JDBC pratique
 *
 * @author $author$
 * @version $Revision$
  */
@Slf4j
public class Pratique {
	public static void main(String[] args) {
		SingleConnectionDataSource dataSource = new SingleConnectionDataSource();
		dataSource.setDriverClassName("org.h2.Driver");
		dataSource.setUrl("jdbc:h2:~/testp");
		dataSource.setUsername("sa");
		dataSource.setPassword("");

		JdbcTemplate jdbcTemplate = new JdbcTemplate(dataSource);
		jdbcTemplate.execute("drop TABLE IF EXISTS utilisateur");
		jdbcTemplate
				.execute("create table utilisateur (id int, nom varchar, prenom varchar)");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (1, 'A', 'a')");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (2, 'B', 'b')");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (3, 'C', 'c')");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (4, 'D', 'd')");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (5, 'E', 'e')");
		jdbcTemplate
				.execute("insert into utilisateur (id, nom, prenom) values (6, 'F', 'f')");

		List<Utilisateur> utilisateurs= jdbcTemplate.query( "select nom, prenom from utilisateur", new UtilisateurMapper());
		    
		log.info("Utilisateurs:");
		utilisateurs.forEach(util -> log.info(util.toString()));

		
		jdbcTemplate.execute("delete from utilisateur");
		jdbcTemplate.execute("drop table utilisateur");

		dataSource.destroy();
	}

}
