package fr.eni;

import lombok.extern.java.Log;
import lombok.extern.slf4j.Slf4j;

import org.apache.commons.dbcp.BasicDataSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.BeanPropertySqlParameterSource;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * DOCUMENT ME!
 *
 * @author $author$
 * @version $Revision$
  */
@Slf4j
public class JdbcUtilisateurDao {
  /**
   * DOCUMENT ME!
   */
  private BasicDataSource dataSource;

  /**
   * DOCUMENT ME!
   */
  private JdbcTemplate jdbcTemplate;

  /**
   * DOCUMENT ME!
   */
  private SimpleJdbcInsert insertUtilisateur;

  /**
   * Creates a new JdbcUtilisateurDao object.
   */
  public JdbcUtilisateurDao() {
    super();
  }

  /**
   * DOCUMENT ME!
   *
   * @param dataSource DOCUMENT ME!
   */
  @Autowired
  public void setDataSource(BasicDataSource dataSource) {
    log.info("par setter setDataSource");
    this.dataSource = dataSource;
    this.jdbcTemplate = new JdbcTemplate(dataSource);
    this.insertUtilisateur = new SimpleJdbcInsert(dataSource).withTableName(
        "Utilisateur").usingColumns("nom", "prenom")
                                                             .usingGeneratedKeyColumns("id");
  }

  /**
   * DOCUMENT ME!
   *
   * @param utilisateur DOCUMENT ME!
   */
  public void ajouter(Utilisateur utilisateur) {
    Map<String, Object> parameters = new HashMap<String, Object>(3);
    parameters.put("nom", utilisateur.getNom());
    parameters.put("prenom", utilisateur.getPrenom());
    insertUtilisateur.execute(parameters);
  }

  /**
   * DOCUMENT ME!
   *
   * @param utilisateur DOCUMENT ME!
   */
  public void ajouter2(Utilisateur utilisateur) {
    Map<String, Object> parameters = new HashMap<String, Object>(2);
    parameters.put("nom", utilisateur.getNom());
    parameters.put("prenom", utilisateur.getPrenom());
    parameters.put("id", utilisateur.getId());

    Number newId = insertUtilisateur.executeAndReturnKey(parameters);
    utilisateur.setId(newId.longValue());
  }

  /**
   * DOCUMENT ME!
   *
   * @param utilisateur DOCUMENT ME!
   */
  public void ajouter3(Utilisateur utilisateur) {
    SqlParameterSource parameters = new BeanPropertySqlParameterSource(utilisateur);
    Number newId = insertUtilisateur.executeAndReturnKey(parameters);
    utilisateur.setId(newId.longValue());
  }

  /**
   * DOCUMENT ME!
   *
   * @param utilisateur DOCUMENT ME!
   */
  public void ajouter4(Utilisateur utilisateur) {
    SqlParameterSource parameters = new MapSqlParameterSource().addValue("nom",
        utilisateur.getNom()).addValue("prenom", utilisateur.getPrenom());
    Number newId = insertUtilisateur.executeAndReturnKey(parameters);
    utilisateur.setId(newId.longValue());
  }

  /**
   * DOCUMENT ME!
   *
   * @return DOCUMENT ME!
   */
  public List<Utilisateur> getAll() {
    List<Utilisateur> utilisateurs = jdbcTemplate.query("select nom, prenom, id from utilisateur",
        new RowMapper<Utilisateur>() {
          public Utilisateur mapRow(ResultSet rs, int rowNum)
            throws SQLException {
            Utilisateur utilisateur = new Utilisateur();
            utilisateur.setNom(rs.getString("nom"));
            utilisateur.setPrenom(rs.getString("prenom"));
            utilisateur.setId(rs.getInt("id"));

            return utilisateur;
          }
        });

    return utilisateurs;
  }
}
