package fr.eni.ep3jasp.chap12.hateoas.ex1.domain;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Builder;
import lombok.NoArgsConstructor;

import javax.persistence.*;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import java.util.Objects;

// tag::code[]
@Entity
@Table(name = "region")
@Builder
@NoArgsConstructor
public class Region implements Serializable {

  private static final long serialVersionUID = 1L;

  //Constructeur
  public Region(Long id, String code, String nom, Set<Departement> departements) {
    this.id = id;
    this.code = code;
    this.nom = nom;
    this.departements = departements;
  }

  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  private Long id;

  @Column(name = "code")
  private String code;

  @Column(name = "nom")
  private String nom;

  @OneToMany(mappedBy = "region")
  @JsonIgnore
  private Set<Departement> departements = new HashSet<>();

  public Long getId() {
    return id;
  }

  public void setId(Long id) {
    this.id = id;
  }

  public String getCode() {
    return code;
  }

  public Region code(String code) {
    this.code = code;
    return this;
  }

  public void setCode(String code) {
    this.code = code;
  }

  public String getNom() {
    return nom;
  }

  public Region nom(String nom) {
    this.nom = nom;
    return this;
  }

  public void setNom(String nom) {
    this.nom = nom;
  }

  public Set<Departement> getDepartements() {
    return departements;
  }

  public Region departements(Set<Departement> departements) {
    this.departements = departements;
    return this;
  }

  public Region addDepartement(Departement departement) {
    this.departements.add(departement);
    departement.setRegion(this);
    return this;
  }

  public Region removeDepartement(Departement departement) {
    this.departements.remove(departement);
    departement.setRegion(null);
    return this;
  }

  public void setDepartements(Set<Departement> departements) {
    this.departements = departements;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    Region region = (Region) o;
    if (region.getId() == null || getId() == null) {
      return false;
    }
    return Objects.equals(getId(), region.getId());
  }

  @Override
  public int hashCode() {
    return Objects.hashCode(getId());
  }

  @Override
  public String toString() {
    return "Region{" +
          "id=" + getId() +
          ", code='" + getCode() + "'" +
          ", nom='" + getNom() + "'" +
          "}";
  }
}
// end::code[]