package fr.eni.ep3jasp.messages.redis;

import java.util.concurrent.CountDownLatch;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;

@SpringBootApplication
public class Ex5MessagerieRedis implements CommandLineRunner {

  private static final Logger LOGGER = LoggerFactory.getLogger(Ex5MessagerieRedis.class);

  public static void main(String[] args) {
    SpringApplication.run(Ex5MessagerieRedis.class, args);
  }

  @Autowired
  StringRedisTemplate stringRedisTemplate;

  @Autowired
  CountDownLatch compteur;

  @Override
  public void run(String... args) throws Exception {
    LOGGER.info("Sending message...");
    stringRedisTemplate.convertAndSend("messagerie", "Démarrage de l'application");
    stringRedisTemplate.convertAndSend("messagerie", "Exécution des traitements");
    stringRedisTemplate.convertAndSend("messagerie", "Fin des traitements");
    compteur.await();
    System.exit(0);
  }


  @Bean
  StringRedisTemplate template(RedisConnectionFactory connectionFactory) {
    return new StringRedisTemplate(connectionFactory);
  }

  @Bean
  MessageListenerAdapter listenerAdapter(RecepteurMessages recepteurMessages) {
    return new MessageListenerAdapter(recepteurMessages, "receiveMessage");
  }

  @Bean
  RedisMessageListenerContainer conteneur(RedisConnectionFactory connectionFactory,
                                          MessageListenerAdapter listenerAdapter) {
    RedisMessageListenerContainer container = new RedisMessageListenerContainer();
    container.setConnectionFactory(connectionFactory);
    container.addMessageListener(listenerAdapter, new PatternTopic("messagerie"));
    return container;
  }

  @Bean
  RecepteurMessages recepteur(CountDownLatch compteur) {
    return new RecepteurMessages(compteur);
  }

  @Bean
  CountDownLatch compteur() {
    return new CountDownLatch(3);
  }

  class RecepteurMessages {
    private final Logger LOGGER = LoggerFactory.getLogger(RecepteurMessages.class);

    private CountDownLatch latch;

    @Autowired
    public RecepteurMessages(CountDownLatch latch) {
      this.latch = latch;
    }

    public void receiveMessage(String message) {
      LOGGER.info("Reception du message [{}]",message);
      latch.countDown();
    }
  }
}
