package fr.eni.ep3jasp.chap05;

import org.springframework.batch.core.Job;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.configuration.annotation.EnableBatchProcessing;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.item.database.BeanPropertyItemSqlParameterSourceProvider;
import org.springframework.batch.item.database.JdbcBatchItemWriter;
import org.springframework.batch.item.file.FlatFileItemReader;
import org.springframework.batch.item.file.mapping.BeanWrapperFieldSetMapper;
import org.springframework.batch.item.file.mapping.DefaultLineMapper;
import org.springframework.batch.item.file.transform.DelimitedLineTokenizer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.io.ClassPathResource;

import javax.sql.DataSource;

// tag::code[]
@Configuration
@EnableBatchProcessing
public class BatchConfiguration {

  @Autowired
  public JobBuilderFactory jobBuilderFactory;

  @Autowired
  public StepBuilderFactory stepBuilderFactory;

  @Autowired
  public DataSource dataSource;

  // tag::readerwriterprocessor[]
  @Bean
  public FlatFileItemReader<Departement> reader() {
    FlatFileItemReader<Departement> reader = new FlatFileItemReader<Departement>();
    reader.setResource(new ClassPathResource("departements.csv"));
    reader.setLineMapper(new DefaultLineMapper<Departement>() {{
      setLineTokenizer(new DelimitedLineTokenizer() {{
        setNames(new String[]{"code", "nom"});
      }});
      setFieldSetMapper(new BeanWrapperFieldSetMapper<Departement>() {{
        setTargetType(Departement.class);
      }});
    }});
    return reader;
  }

  @Bean
  public DepartementItemProcessor processor() {
    return new DepartementItemProcessor();
  }

  @Bean
  public JdbcBatchItemWriter<Departement> writer() {
    JdbcBatchItemWriter<Departement> writer = new JdbcBatchItemWriter<Departement>();
    writer.setItemSqlParameterSourceProvider(new BeanPropertyItemSqlParameterSourceProvider<Departement>());    writer.setSql("INSERT INTO departement (code, nom) VALUES (:code, :nom)");
    writer.setDataSource(dataSource);
    return writer;
  }
  // end::readerwriterprocessor[]

  // tag::jobstep[]
  @Bean
  public Job importUserJob(JobCompletionNotificationListener listener) {
    return jobBuilderFactory.get("importUserJob")
          .incrementer(new RunIdIncrementer())
          .listener(listener)
          .flow(step1())
          .end()
          .build();
  }

  @Bean
  public Step step1() {
    return stepBuilderFactory.get("step1")
          .<Departement, Departement>chunk(10)
          .reader(reader())
          .processor(processor())
          .writer(writer())
          .build();
  }
  // end::jobstep[]
}
// end::code[]